/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* Loading and providing of different properties stored in XML files. */
#ifndef PROPERTIES_H
#define PROPERTIES_H



/* List of different properties types which correspond to properties files. */
typedef enum PropTypeEnum
{
    /* Properties file storing various strings. */
    PropStrings,

    /* Properties file storing application settings. */
    PropSettings,

    /* add new values above */
    NumberOfPropTypes
} PropType;




/* Properties file storing application settings. */
#define PROP_SETTINGS "settings.xml"




/* Initializes the properties module. */
void prop_initialize(void);


/* Cleans up the properties module. */
void prop_clean_up(void);



/* Returns the value corresponding to 'key' from the properties file 'type'.
 * If there is no such value then NULL is returned. */
G_CONST_RETURN gchar* prop_get(PropType type, const gchar *key);


/* Returns the value corresponding to 'key' for 'PropStrings'.
 * If there is no such value then 'key' is returned. */
G_CONST_RETURN gchar* _T(const gchar *key);



/* Returns the directory where to store user-defined matrices. */
G_CONST_RETURN gchar* prop_get_matrices_directory(void);


/* Returns the maximum number of rows and columns of matrices handled
 * in this program. All matrices in this program have this size. */
guint prop_get_max_matrix_size(void);


/* Returns the GDK name of the highlight color for matrix cells. */
G_CONST_RETURN gchar* prop_get_matrix_highlight_color(void);




#endif /* PROPERTIES_H */
