/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef VIEW_PRIVATE_H
#define VIEW_PRIVATE_H

#include "View.h"




/* see view.main.c for the following functions */


/* Creates the main window and its components. */
void v_init_main_window(void);

/* Creates the main panel and its sub-panels holding the visible elements. */
void v_init_main_panel(GtkContainer *mainWindow);


/* Returns the main window. */
GtkWindow* v_get_main_window(void);




/* see view.menu.c for the following function */


/* Creates the menu. */
void v_init_menu(GtkBox *mainPanel);




/* see view.help.c for the following function */


/* Create a window showing some general program information. */
void v_show_about_dialog(void);




/* see view.algorithm.c for the following functions */


/* Creates the algorithm panel holding all information of an algorithm.
 * Description and pseudo-code are not displayed at the same time. Call
 * 'v_show_algorithm_description()' to activate one of them. */
void v_init_algorithm_panel(GtkBox *topPanel);


/* Used to switch to either the description or the pseudo-code of an
 * algorithm. Only of of them can be displayed at a time. */
void v_show_algorithm_description(gboolean showDescription);




/* see view.algorithm.dialog.c for the following function */


/* Shows a dialog in which the user can choose an algorithm from a list.
 * Description and pseudo-code are displayed for the currently selected
 * algorithm. */
void v_show_algorithm_dialog(void);




/* see view.semiring.c for the following functions */


/* Creates the semiring panel holding all information of a semiring. */
void v_init_semiring_panel(GtkBox *topPanel);




/* see view.semiring.dialog.c for the following function */


/* Shows a dialog in which the user can choose a semiring from a list. */
void v_show_semiring_dialog(void);




/* see view.matrix.c for the following functions */


/* Creates the matrix panel holding all information of the computation and
 * its matrices. */
void v_init_matrix_panel(GtkBox *bottomPanel);


/* Executes the computation.
 *
 * stepwise : TRUE if all intermediate steps should also be computed,
 *            otherwise FALSE.
 */
void v_compute(gboolean stepwise);





/* see view.matrix.dialog.c for the following function */


/* Shows a dialog in which the user can choose a matrix from a list or add
 * a new matrix. All matrices can be edited and removed. */
void v_show_matrix_dialog(void);




/* see view.utils.c for the following functions */


/* Initializes the utilities module. */
void v_utils_initialize(void);



/* Returns the global text line height (for the standard font). */
guint v_get_line_height(void);


/* Sets the font family and the wrap mode of 'textview'.
 * Pass TRUE if a monospaced font and no wrap should be used in 'textview'.
 * Otherwise a sans serif font and word wrap is set. */
void v_set_monospace_font(GtkTextView *textview, gboolean monospace);


/* Creates a static text field in 'panel' with 'lines' lines and a width of
 * 'width' pixels. */
GtkTextView *v_create_text_field(GtkBox *panel, guint width, guint lines);



/* Sets the tooltip 'tip' for 'widget'. */
void v_set_tooltip(GtkWidget *widget, const gchar *tip);


/* Enables or disables all tooltips set by 'v_set_tooltip()'. */
void v_show_tooltips(gboolean show);



/* Returns the first index of selected items found in 'selection'. */
guint v_get_selected_list_item_index(GtkTreeSelection *selection);



#endif /* VIEW_PRIVATE_H */
