/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "View.Private.h"



/* The main window of the application. */
static GtkWidget *_mainWindow;



/* Adapter for correctly handling the signal. */
static void _on_destroy(GtkObject *object, gpointer notUsed)
{
    gtk_main_quit();
}





/* Returns the main window. */
GtkWindow* v_get_main_window(void)
{
    return GTK_WINDOW(_mainWindow);
}



/* Creates the main window and its components. */
void v_init_main_window(void)
{
    _mainWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);

    /* settings for the main window */
    gtk_window_set_title(GTK_WINDOW(_mainWindow),
                         "The Algebraic Path Problem Calculator");
    gtk_window_set_resizable(GTK_WINDOW(_mainWindow), FALSE);
    gtk_container_set_border_width(GTK_CONTAINER(_mainWindow), 0);

    g_signal_connect(G_OBJECT(_mainWindow), "destroy",
                     G_CALLBACK(_on_destroy), NULL);


    /* create components */
    v_init_main_panel(GTK_CONTAINER(_mainWindow));


    gtk_widget_show(_mainWindow);
}



/* Creates the main panel and its sub-panels holding the visible elements. */
void v_init_main_panel(GtkContainer *mainWindow)
{
    GtkWidget *mainPanel;
    GtkWidget *topPanel;
    GtkWidget *algorithmPanel;
    GtkWidget *emptyPanel;
    GtkWidget *semiringPanel;
    GtkWidget *bottomPanel;


    mainPanel = gtk_vbox_new(FALSE, 0);
    gtk_container_add(mainWindow, mainPanel);


    /* create menu */
    v_init_menu(GTK_BOX(mainPanel));


    /* create the top panel holding the algorithm and the semiring section */
    topPanel = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(mainPanel), topPanel, FALSE, FALSE, 12);


    /* create the algorithm panel as sub-panel of the top panel */
    algorithmPanel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(topPanel), algorithmPanel, FALSE, FALSE, 12);

    v_init_algorithm_panel(GTK_BOX(algorithmPanel));

    gtk_widget_show(algorithmPanel);


    /* create an empty panel (for a better layout) */
    emptyPanel = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(topPanel), emptyPanel, FALSE, FALSE, 6);
    gtk_widget_show(emptyPanel);


    /* create the semiring panel as sub-panel of the top panel */
    semiringPanel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(topPanel), semiringPanel, FALSE, FALSE, 12);

    v_init_semiring_panel(GTK_BOX(semiringPanel));

    gtk_widget_show(semiringPanel);


    gtk_widget_show(topPanel);



    /* create the bottom panel holding the matrices */
    bottomPanel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(mainPanel), bottomPanel, FALSE, FALSE, 12);

    v_init_matrix_panel(GTK_BOX(bottomPanel));

    gtk_widget_show(bottomPanel);


    gtk_widget_show(mainPanel);
}
