/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.h"
#include "View.Private.h"



/* the button with the label "Select semiring". */
static GtkWidget *_changeButton;


/* holds the name of the currently selected semiring */
static GtkTextBuffer *_nameBuffer;

/* holds the description of the currently selected semiring */
static GtkTextBuffer *_descriptionBuffer;



/* Called whenever the currently selected algorithm has changed. */
static void _on_algorithm_changed(gpointer notUsed)
{
    gtk_widget_set_sensitive(_changeButton,
                             m_algorithms_get_selected() != NULL);
}


/* Called whenever the currently selected semiring has changed. */
static void _on_semiring_changed(gpointer notUsed)
{
    Semiring semiring = m_semirings_get_selected();

    /* if there is no semiring currently selected */
    if (semiring == NULL)
    {
        gtk_text_buffer_set_text(_nameBuffer, _T("No semiring selected."), -1);
        gtk_text_buffer_set_text(_descriptionBuffer,
                                 _T("No semiring description available."),
                                 -1);
    }
    else    /* a semiring is selected */
    {
        gtk_text_buffer_set_text(_nameBuffer, semiring->name, -1);
        gtk_text_buffer_set_text(_descriptionBuffer, semiring->description, -1);
    }
}



/* Adapter for correctly handling the signal. */
static void _on_change_semiring_clicked(GtkButton *button, gpointer notUsed)
{
    v_show_semiring_dialog();
}


/* Creates the semiring panel holding all information of a semiring. */
void v_init_semiring_panel(GtkBox *semiringPanel)
{
    GtkWidget *panel;
    GtkWidget *label;
    GtkWidget *buttonPanel;
    GtkTextView *view;
    gchar *string;



    /* create the title section */
    panel = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(semiringPanel, panel, FALSE, FALSE, 0);

    label = gtk_label_new("");
    string = g_strconcat("<b><big>", _T("2. Semiring"), "</big></b>", NULL);
    gtk_label_set_markup(GTK_LABEL(label), string);
    g_free(string);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(GTK_BOX(panel), label, FALSE, FALSE, 0);
    gtk_widget_show(label);

    buttonPanel = gtk_hbutton_box_new();
    gtk_button_box_set_layout(GTK_BUTTON_BOX(buttonPanel), GTK_BUTTONBOX_START);
    gtk_box_pack_end(GTK_BOX(panel), buttonPanel, FALSE, FALSE, 6);

    _changeButton = gtk_button_new_with_mnemonic(_T("Select semiring ..."));
    gtk_widget_set_sensitive(_changeButton, FALSE);
    v_set_tooltip(_changeButton,
                  _T("Press this button to select another semiring."));
    gtk_container_add(GTK_CONTAINER(buttonPanel), _changeButton);
    g_signal_connect(G_OBJECT(_changeButton), "clicked",
                     G_CALLBACK(_on_change_semiring_clicked), NULL);
    gtk_widget_show(_changeButton);

    gtk_widget_show(buttonPanel);


    gtk_widget_show(panel);



    /*  create the name section */
    panel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(semiringPanel, panel, FALSE, FALSE, 6);

    label = gtk_label_new(_T("Semiring name:"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(GTK_BOX(panel), label, FALSE, FALSE, 6);
    gtk_widget_show(label);

    view = v_create_text_field(GTK_BOX(panel), 350, 1);
    _nameBuffer = gtk_text_view_get_buffer(view);
    gtk_text_buffer_set_text(_nameBuffer, _T("No semiring selected."), -1);

    gtk_widget_show(panel);



    /*  create the description section */
    panel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(semiringPanel, panel, FALSE, FALSE, 6);

    label = gtk_label_new(_T("Semiring description:"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(GTK_BOX(panel), label, FALSE, FALSE, 6);
    gtk_widget_show(label);

    view = v_create_text_field(GTK_BOX(panel), 300, 13);
    _descriptionBuffer = gtk_text_view_get_buffer(view);
    gtk_text_buffer_set_text(_descriptionBuffer,
                             _T("No semiring description available."),
                             -1);

    gtk_widget_show(panel);



    /* add callback functions to stay informed whenever the selected
     * algorithm or semiring changes */
    m_algorithms_add_hook(OnActiveItemHasChanged, _on_algorithm_changed);
    m_semirings_add_hook(OnActiveItemHasChanged, _on_semiring_changed);
}
