/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/* The definition of the matrix structure. */

#ifndef MATRIX_H
#define MATRIX_H



/* Include needed files. */
#include "AppC.h"
#include "Semiring.h"



/* The structure of a matrix. */
typedef struct MatrixStruct
{
    /* The filename of the matrix (without its ending). */
    gchar *name;

    /* The size (=number of rows and columns) of this matrix. */
    guint size;

    /* The matrix itself.
     * For indexing an item use the following semantic: matrix[row][column]. */
    SemiringItem** matrix;

    /* The according semiring. */
    Semiring semiring;
} *Matrix;



/* A collection of matrix functions (used in the algorithms). */
typedef struct MatrixFunctionsStruct
{
    /* Creates a new matrix. */
    Matrix (*create_matrix)(void);

    /* Copies matrix 'source' to 'destination'. */
    void (*copy_matrix)(Matrix destination, const Matrix source);

    /* Copies matrix 'source' to 'destination'. */
    void (*free_matrix)(Matrix matrix);
} MatrixFunctions;


#endif /* MATRIX_H */
