/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "AppC.h"



static SemiringItem zero(void)
{
    return GUINT_TO_POINTER(0);
}

static SemiringItem one(void)
{
    return GUINT_TO_POINTER(1);
}

static SemiringItem star(SemiringItem a)
{
    return one();
}

static SemiringItem add(SemiringItem a, SemiringItem b)
{
    guint _a = GPOINTER_TO_UINT(a);
    guint _b = GPOINTER_TO_UINT(b);
    return GUINT_TO_POINTER(_a + _b);
}

static SemiringItem mult(SemiringItem a, SemiringItem b)
{
    guint _a = GPOINTER_TO_UINT(a);
    guint _b = GPOINTER_TO_UINT(b);
    return GUINT_TO_POINTER(_a * _b);
}


static gboolean equals(SemiringItem a, SemiringItem b)
{
    return a == b;
}


static SemiringItem copy(SemiringItem item)
{
    return item;
}


static SemiringItem from_string(const gchar *string, guint *read)
{
    guint result;
    const gchar *p;
    gchar *end;

    p = string;
    if (read != NULL) *read = 0;

    if (*p == '-') return zero();
    result = strtoul(p, &end, 10);
    if (result == ULONG_MAX || end == p) return zero();

    if (read != NULL) *read = end - string;

    return GUINT_TO_POINTER(result);
}


static gchar* to_string(SemiringItem item)
{
    return g_strdup_printf("%u", GPOINTER_TO_UINT(item));
}


static void release(SemiringItem item)
{
    /* do nothing in here */
}





G_MODULE_EXPORT void semiring_initialize(Semiring sr)
{
    sr->zero = zero;
    sr->one = one;
    sr->star = star;
    sr->add = add;
    sr->mult= mult;

    sr->equals = equals;
    sr->copy = copy;

    sr->from_string = from_string;
    sr->to_string = to_string;

    sr->release = release;
}
