/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "AppC.h"
#include "Fraction.h"



static SemiringItem zero(void)
{
    return (SemiringItem) fraction_zero();
}


static SemiringItem one(void)
{
    return (SemiringItem) fraction_one();
}


static SemiringItem star(SemiringItem a)
{
    return one();
}


static SemiringItem add(SemiringItem a, SemiringItem b)
{
    return (SemiringItem) fraction_max((Fraction) a, (Fraction) b);
}


static SemiringItem mult(SemiringItem a, SemiringItem b)
{
    return (SemiringItem) fraction_mul((Fraction) a, (Fraction) b);
}



static gboolean equals(SemiringItem a, SemiringItem b)
{
    return fraction_equals((Fraction) a, (Fraction) b);
}


static SemiringItem copy(SemiringItem item)
{
    return fraction_copy((Fraction) item);
}


static SemiringItem from_string(const gchar *string, guint *read)
{
    Fraction result = fraction_from_string(string, read);

    Fraction zero = fraction_zero();
    Fraction one = fraction_one();


    /* check if 0 <= 'result' <= 1 holds */
    if (fraction_equals(fraction_max(result, one), one) &&
        fraction_equals(fraction_min(result, zero), zero))
    {
        fraction_free(zero);
        fraction_free(one);
        
        return (SemiringItem) result;
    }
    else   /* 'result' is not within the interval [0, 1] */
    {
        fraction_free(one);
        fraction_free(result);

        if (read != NULL) *read = 0;
        return (SemiringItem) zero;
    }
}


static char* to_string(SemiringItem item)
{
    return fraction_to_string((Fraction) item);
}


static void release(SemiringItem item)
{
    fraction_free((Fraction) item);
}



G_MODULE_EXPORT void semiring_initialize(Semiring sr)
{
    sr->zero = zero;
    sr->one = one;
    sr->star = star;
    sr->add = add;
    sr->mult= mult;

    sr->equals = equals;
    sr->copy = copy;

    sr->from_string = from_string;
    sr->to_string = to_string;

    sr->release = release;
}
